# AssemblyScript development environment for SORACOM Orbit

## Setup Development Environment

1. Run `init` NPM script defined in `package.json`.
   - Open NPM Scripts View and run `init`, or
   - Open integrated terminal and run `npm run init`

## Development

See developer's guide for detail. Don't forget to click `Reopen in Container` when you launch the workspace going forward.

1. Open and edit [`assembly/index.ts`](assembly/index.ts) which is your AssemblyScript for SORACOM Orbit.
2. Run `build` script to build WASM module under [`build/`](build) directory.
   - Open NPM Scripts View and run `build`, or
   - Open integrated terminal and run `npm run build`, or
   - Trigger **Run Build Task** from the VS Code **Command Palette** (<kbd>⇧⌘P</kbd>), or
   - <kbd>⇧⌘B</kbd>

Please note that you have to export `uplink()` and/or `downlink()` in your AssemblyScript.

```ts
// For processing uplink (UE to SORACOM)
export function uplink(): i32 {
  /* code  */
}

// For processing downlink (SORACOM to UE)
export function downlink(): i32 {
  /* code */
}
```

### Configuration

#### Environment Variables

If you set following environment variables, these will be exposed to the container, VS Code, and related sub-processes (terminals, tasks, etc.):

- `SORACOM_AUTHKEY_ID`
- `SORACOM_AUTHKEY_SECRET`
- `SORACOM_COVERAGE`

#### `package.json`

You can specify Orbit configuration at [`package.json`](package.json) as follows:

```json5
  "soralet": { // SORACOM Orbit configuration
    "auth": { // SORACOM API authentication information
      "coverageType": "env", // SORACOM coverage, 'g' for global and 'jp' for Japan. Specify 'env' to use SORACOM_COVERAGE environment variable.
      "authKeyId": "env", // SORACOM API auth key ID. Specify 'env' to use SORACOM_AUTHKEY_ID environment variable.
      "authKey": "env" // Specify auth key ID starts with 'secret-', or 'env'
    },
    "config": { // SORACOM Orbit configuration information
      "projectId": "soralet", // A project ID for the soralet.
      "version": 1, // "A version number of the soralet.
      "contentType": "application/octet-stream", // MIME type of the body. 'application/json', 'application/octet-stream', or 'text/csv'
      "direction": "uplink", // A direction of the data which the soralet want to work with. Will be an entry point (function name) of the soralet. 'uplink', 'downlink', or 'both'
      "useLocation": false, // Whether to use SORACOM location service, only available with plan-KM1.
      "useMetadata": false // Whether to use SORACOM metadata service.
    }
  }
```

The setting is not used at this moment, but will be used for deployment or configuration update.

## Testing

As a starter, [Jest](https://jestjs.io/) [snapshot testing](https://jestjs.io/docs/en/snapshot-testing) is configured, but you can use any test framework or just [`assert` module](https://nodejs.org/api/assert.html).

1. Add your test cases to [`tests/inputs.json`](tests/inputs.json).
2. Run `test` script. The first time this test is run, Jest creates a snapshot under [`tests/__snapshots__/`](tests/__snapshots__) directory. Review result and fix your implementation if needed.
   - Open NPM Scripts View and run `test`, or
   - Open integrated terminal and run `npm test`, or
   - Trigger **Run Test Task** from the VS Code **Command Palette** (<kbd>⇧⌘P</kbd>)
3. Run `test:updateSnapshot` to update snapshot artifact.

## Deployment

1. Run `build:optimized` script to build optimized WASM module under [`build/`](build) directory.
2. See developer's guide.

## Limitations

- You cannot use some of AssemblyScript standard library, especially which is relied on host support.
  - [Date | The AssemblyScript Book](https://www.assemblyscript.org/stdlib/date.html)
  - [Math | The AssemblyScript Book](https://www.assemblyscript.org/stdlib/math.html)

## References

- [Introduction - The AssemblyScript Book](https://docs.assemblyscript.org/)
- [Developing inside a Container using Visual Studio Code Remote Development](https://code.visualstudio.com/docs/remote/containers)
- [Getting Started · Jest](https://jestjs.io/docs/en/getting-started.html)
